(document:surround "/std/frame")
(document:insert "/std/functions")
(document:insert "/standalone/profile")

;;;;;;;;;;;;;

;;for main qt-browser
(document:envelop with-container-presentations
                  ((wizard-face '(/std/primitive type "wizardface")) ))

(document:envelop with-translation _ "alterator-wizard")

(define (make-wizard-face . args)
  (document:id wizard-id (apply wizard-face args)))

;;;;;;;;;;;;;;
(define *step* (make-cell 0))

(define (step) (cell-ref *step*))
(define (++step) (cell-set! *step* (+ (step) 1)) (step))
(define (--step) (cell-set! *step* (- (step) 1)) (step))
(define (last) (- (length (cell-ref profile-modules)) 1))

;;register globals
(set-global! 'frame:next (make-cell #f))
(set-global! 'frame:back (make-cell #f))
;;get globals to avoid to be context specific
(define next-cb (global 'frame:next))
(define back-cb (global 'frame:back))

(define (call-cell cell)
  (let ((proc (cell-ref cell)))
    (and (procedure? proc) (proc))))

(define (popup-overflow proc cb)
  (document:popup-critical
   (string-append "Please remove " proc " call from " cb " function"))
   'cancel)

(define (sure-yes key name list)
  (or (member key list)
      (wizard-id action-add (vector key name))))

(define (sure-no key name list)
  (and (member key list)
       (wizard-id action-remove key)))

(define (action-name x) (vector-ref x 0))

(define (check-buttons step last)
  (let ((buttons (map action-name (wizard-id actions))))
  (cond
   ((= step 0)
    (sure-yes 'forward (_ "Next") buttons)
    (sure-no 'backward (_ "Previous") buttons)
    (sure-no 'finish (_ "Finish") buttons))
   ((< 0 step last)
    (sure-yes 'forward (_ "Next") buttons)
    (sure-yes 'backward (_ "Previous") buttons)
    (sure-no 'finish (_ "Finish") buttons))
   ((= step last)
    (sure-no 'forward (_ "Next") buttons)
    (sure-yes 'backward (_ "Previous") buttons)
    (sure-yes 'finish (_ "Finish") buttons))))
    #t)

(define (go-next)
  ;;protect from stack overflow
  (frame:set-next (thunk (popup-overflow "frame:next" "frame:on-next")))
  (let ((last (last)))
    (and (< (step) last)
         (not (eq? (call-cell next-cb) 'cancel))
         (wizard-id current-step (++step))
         (check-buttons (step) last)
         (frame:replace (list-ref (cell-ref profile-modules) (step))
                        'frame:direction 'next)))
  (frame:set-next go-next))

(define (go-back)
  ;;protect from stack overflow
  (frame:set-back (thunk (popup-overflow "frame:back" "frame:on-back")))
  (let ((last (last)))
    (and (> (step) 0)
         (not (eq? (call-cell back-cb) 'cancel))
         (wizard-id current-step (--step))
         (check-buttons (step) last)
         (frame:replace (list-ref (cell-ref profile-modules) (step))
                        'frame:direction 'back)))
  (frame:set-back go-back))

(define (step-names)
  (define-operation get-lang)
  (let* ((langlist (get-lang (fluid-ref generic-session)))
         (pkg (cell-ref profile-translation))
         (tr (and pkg (make-translator pkg langlist))))
    (map (lambda (x)
           (let* ((info (woo-read-first (string-append "/atlas/" x)))
                  (desc (woo-get-option info 'acc-description x))
                  (icon (woo-get-option info 'acc-icon ""))
                  (desc (if tr (tr desc) desc)))
             (cons desc icon)))
         (cell-ref profile-modules))))

;;;;;;;;;;;;;;;;;
(frame:set-next go-next)
(frame:set-back go-back)

(load-profile (global 'profile-name))

(let ((modules (cell-ref profile-modules)))
  (if (not-pair? modules)
      (label (_ "Fatal: no steps was found"))
      (make-wizard-face
       layout-policy 100 -2
       title (or (cell-ref profile-full-name) (_ "Wizard"))
       steps (step-names)
       current-step 0
       (frame:make-page (car modules)))))

(set-global! 'frame:wizard wizard-id)

;;;;;;;;;;;;;;;;
(apply-actions
 wizard-id
 ;; `(abort ,(_ "Abort")) (thunk
 ;;                       (and (eq? 'ok (document:popup-warning (_ "Are you really want to abort installation?") 'cancel 'ok))
 ;;                            (document:end)))
 ;;actions
 'finish (thunk (or (eq? 'cancel (call-cell next-cb))
                    (document:end)))
 'forward (thunk (frame:next))
 'backward (thunk (frame:back)) )

;;first step buttons state
(check-buttons 0 (last))

;;emulate first time loading instructions
(document:root (when loaded (page-id loaded)))
